<?php
/**
 * Shopware 5
 * Copyright (c) shopware AG
 *
 * According to our dual licensing model, this program can be used either
 * under the terms of the GNU Affero General Public License, version 3,
 * or under a proprietary license.
 *
 * The texts of the GNU Affero General Public License with an additional
 * permission and of our proprietary license can be found at and
 * in the LICENSE file you have received along with this program.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 *
 * "Shopware" is a registered trademark of shopware AG.
 * The licensing of the program under the AGPLv3 does not imply a
 * trademark license. Therefore any rights, title and interest in
 * our trademarks remain entirely with us.
 */

/**
 * @category  Shopware
 * @package   Shopware\Tests
 * @copyright Copyright (c) 2012, shopware AG (http://www.shopware.de)
 */
class Shopware_Tests_Controllers_Backend_ProductFeedTest extends Enlight_Components_Test_Controller_TestCase
{
    /**
     * feed dummy data
     *
     * @var array
     */
    private $feedData = array(
        'name'=>'UnitTest Produktsuche',
        'lastExport'=>'2012-06-13 13:45:12',
        'active'=>'1',
        'hash'=>'0805bbb935327228edb5374083b81416',
        'show'=>'1',
        'countArticles'=>'89',
        'expiry'=>'2000-01-01 00:00:00',
        'interval'=>'3456',
        'formatId'=>'2',
        'lastChange'=>'0000-00-00 00:00:00',
        'fileName'=>'export.txt',
        'encodingId'=>'2',
        'categoryId'=>NULL,
        'currencyId'=>'1',
        'customerGroupId'=>'1',
        'partnerId'=>'',
        'languageId'=>NULL,
        'activeFilter'=>'0',
        'imageFilter'=>'0',
        'stockminFilter'=>'0',
        'instockFilter'=>'0',
        'priceFilter'=>'0',
        'ownFilter'=>'',
        'header'=>'{#BOM#}{strip}id{#S#}{/strip}{#L#}',
        'body'=>'',
        'footer'=>'',
        'countFilter'=>'0',
        'shopId'=>'1',
        'variantExport'=>'1'
    );

    /** @var Shopware\Components\Model\ModelManager */
    private $manager = null;

    /**@var $model \Shopware\Models\ProductFeed\ProductFeed*/
    protected $repository = null;

    /**
     * Standard set up for every test - just disable auth
     */
    public function setUp()
    {
        parent::setUp();

        $this->manager    = Shopware()->Models();
        $this->repository = Shopware()->Models()->ProductFeed();

        // disable auth and acl
        Shopware()->Plugins()->Backend()->Auth()->setNoAuth();
        Shopware()->Plugins()->Backend()->Auth()->setNoAcl();
    }

    /**
     * Creates the dummy feed
     *
     * @return \Shopware\Models\ProductFeed\ProductFeed
     */
    private function getDummyFeed()
    {
        $feed = new \Shopware\Models\ProductFeed\ProductFeed();
        $feedData = $this->feedData;
        $feed->fromArray($feedData);
        return $feed;
    }

    /**
     * helper method to create the dummy object
     *
     * @return \Shopware\Models\ProductFeed\ProductFeed
     */
    private function createDummy()
    {
        $feed = $this->getDummyFeed();
        $this->manager->persist($feed);
        $this->manager->flush();

        return $feed;
    }

    /**
     * test the feed list
     */
    public function testGetFeeds()
    {

        //delete old data
        $feeds = $this->repository->findBy(array('hash' => '0805bbb935327228edb5374083b81416'));
        foreach ($feeds as $testFeed) {
            $this->manager->remove($testFeed);
        }
        $this->manager->flush();

        $feed = $this->createDummy();
        /** @var Enlight_Controller_Response_ResponseTestCase */
        $this->dispatch('backend/ProductFeed/getFeeds?page=1&start=0&limit=30');
        $this->assertTrue($this->View()->success);
        $returnData = $this->View()->data;
        $this->assertNotEmpty($returnData);
        $this->assertGreaterThan(0, $this->View()->totalCount);
        $foundDummyFeed = array();
        foreach ($returnData as $feedData) {
            if ($feedData["name"] == $feed->getName()) {
                $foundDummyFeed = $feedData;
            }
        }

        $this->assertEquals($feed->getId(), $foundDummyFeed["id"]);
        $this->manager->remove($feed);
        $this->manager->flush();
    }

    /**
     * test adding a feed
     * @return the id to for the testUpdateFeed Method
     */
    public function testAddFeed()
    {
        $params = $this->feedData;
        $this->Request()->setParams($params);

        $this->dispatch('backend/ProductFeed/saveFeed');
        $this->assertTrue($this->View()->success);
        $this->assertCount(36, $this->View()->data);
        $this->assertEquals($params["name"], $this->View()->data["name"]);

        return $this->View()->data["id"];
    }

    /**
     * test the getDetailFeed Method
     *
     * @depends testAddFeed
     * @param $id
     * @return the id to for the testUpdateFeed Method
     */
    public function testGetDetailFeed($id)
    {
        $params["feedId"] = $id;
        $this->Request()->setParams($params);
        $this->dispatch('backend/ProductFeed/getDetailFeed');
        $this->assertTrue($this->View()->success);
        $returningData = $this->View()->data;
        $dummyFeedData = $this->feedData;
        $this->assertEquals($dummyFeedData["name"],$returningData["name"]);
        $this->assertEquals($dummyFeedData["active"],$returningData["active"]);
        $this->assertEquals($dummyFeedData["hash"],$returningData["hash"]);
        $this->assertEquals($dummyFeedData["countArticles"],$returningData["countArticles"]);
        $this->assertEquals($dummyFeedData["formatId"],$returningData["formatId"]);
        $this->assertEquals($dummyFeedData["fileName"],$returningData["fileName"]);
        $this->assertEquals($dummyFeedData["customerGroupId"],$returningData["customerGroupId"]);
        $this->assertEquals($dummyFeedData["header"],$returningData["header"]);
        $this->assertEquals($dummyFeedData["body"],$returningData["body"]);
        $this->assertEquals($dummyFeedData["footer"],$returningData["footer"]);
        $this->assertEquals($dummyFeedData["shopId"],$returningData["shopId"]);
        return $id;
    }

    /**
     * test updating a feed
     *
     * @depends testGetDetailFeed
     * @param $id
     */
    public function testUpdateFeed($id)
    {
        $params = $this->feedData;
        $params["feedId"] = $id;
        $params["name"] = "phpUnit Test New Name";
        $this->Request()->setParams($params);

        $this->dispatch('backend/ProductFeed/saveFeed');

        $this->assertTrue($this->View()->success);
        $this->assertCount(36, $this->View()->data);
        $this->assertEquals($params["name"], $this->View()->data["name"]);

        return $id;
    }

    /**
     * test delete the feed method
     *
     * @depends testUpdateFeed
     * @param $id
     */
    public function testDeleteFeed($id)
    {
        $params = array();
        $params["id"] = intval($id);
        $this->Request()->setParams($params);
        $this->dispatch('backend/ProductFeed/deleteFeed');
        $this->assertTrue($this->View()->success);
        $this->assertNull($this->repository->find($params["id"]));
    }

    /**
     * test getSuppliers action
     *
     * @return void
     * @internal param $id
     */
    public function testGetSuppliersAction()
    {
        $this->dispatch('backend/ProductFeed/getSuppliers');
        $this->assertTrue($this->View()->success);
        $this->assertGreaterThan(0, $this->View()->total);
    }

    /**
     * test getArticles action
     *
     * @return void
     * @internal param $id
     */
    public function testGetArticlesAction()
    {
        $this->dispatch('backend/ProductFeed/getArticles');
        $this->assertTrue($this->View()->success);
        $this->assertEquals(20, count($this->View()->data));
    }
}
